#ifndef vtk_sqlite_mangle_h
#define vtk_sqlite_mangle_h

#define sqlite3_aggregate_context vtksqlite_sqlite3_aggregate_context
#define sqlite3_aggregate_count vtksqlite_sqlite3_aggregate_count
#define sqlite3_auto_extension vtksqlite_sqlite3_auto_extension
#define sqlite3_backup_finish vtksqlite_sqlite3_backup_finish
#define sqlite3_backup_init vtksqlite_sqlite3_backup_init
#define sqlite3_backup_pagecount vtksqlite_sqlite3_backup_pagecount
#define sqlite3_backup_remaining vtksqlite_sqlite3_backup_remaining
#define sqlite3_backup_step vtksqlite_sqlite3_backup_step
#define sqlite3_bind_blob vtksqlite_sqlite3_bind_blob
#define sqlite3_bind_blob64 vtksqlite_sqlite3_bind_blob64
#define sqlite3_bind_double vtksqlite_sqlite3_bind_double
#define sqlite3_bind_int vtksqlite_sqlite3_bind_int
#define sqlite3_bind_int64 vtksqlite_sqlite3_bind_int64
#define sqlite3_bind_null vtksqlite_sqlite3_bind_null
#define sqlite3_bind_parameter_count vtksqlite_sqlite3_bind_parameter_count
#define sqlite3_bind_parameter_index vtksqlite_sqlite3_bind_parameter_index
#define sqlite3_bind_parameter_name vtksqlite_sqlite3_bind_parameter_name
#define sqlite3_bind_pointer vtksqlite_sqlite3_bind_pointer
#define sqlite3_bind_text vtksqlite_sqlite3_bind_text
#define sqlite3_bind_text16 vtksqlite_sqlite3_bind_text16
#define sqlite3_bind_text64 vtksqlite_sqlite3_bind_text64
#define sqlite3_bind_value vtksqlite_sqlite3_bind_value
#define sqlite3_bind_zeroblob vtksqlite_sqlite3_bind_zeroblob
#define sqlite3_bind_zeroblob64 vtksqlite_sqlite3_bind_zeroblob64
#define sqlite3_blob_bytes vtksqlite_sqlite3_blob_bytes
#define sqlite3_blob_close vtksqlite_sqlite3_blob_close
#define sqlite3_blob_open vtksqlite_sqlite3_blob_open
#define sqlite3_blob_read vtksqlite_sqlite3_blob_read
#define sqlite3_blob_reopen vtksqlite_sqlite3_blob_reopen
#define sqlite3_blob_write vtksqlite_sqlite3_blob_write
#define sqlite3_busy_handler vtksqlite_sqlite3_busy_handler
#define sqlite3_busy_timeout vtksqlite_sqlite3_busy_timeout
#define sqlite3_cancel_auto_extension vtksqlite_sqlite3_cancel_auto_extension
#define sqlite3_changes vtksqlite_sqlite3_changes
#define sqlite3_clear_bindings vtksqlite_sqlite3_clear_bindings
#define sqlite3_close vtksqlite_sqlite3_close
#define sqlite3_close_v2 vtksqlite_sqlite3_close_v2
#define sqlite3_collation_needed vtksqlite_sqlite3_collation_needed
#define sqlite3_collation_needed16 vtksqlite_sqlite3_collation_needed16
#define sqlite3_column_blob vtksqlite_sqlite3_column_blob
#define sqlite3_column_bytes vtksqlite_sqlite3_column_bytes
#define sqlite3_column_bytes16 vtksqlite_sqlite3_column_bytes16
#define sqlite3_column_count vtksqlite_sqlite3_column_count
#define sqlite3_column_decltype vtksqlite_sqlite3_column_decltype
#define sqlite3_column_decltype16 vtksqlite_sqlite3_column_decltype16
#define sqlite3_column_double vtksqlite_sqlite3_column_double
#define sqlite3_column_int vtksqlite_sqlite3_column_int
#define sqlite3_column_int64 vtksqlite_sqlite3_column_int64
#define sqlite3_column_name vtksqlite_sqlite3_column_name
#define sqlite3_column_name16 vtksqlite_sqlite3_column_name16
#define sqlite3_column_text vtksqlite_sqlite3_column_text
#define sqlite3_column_text16 vtksqlite_sqlite3_column_text16
#define sqlite3_column_type vtksqlite_sqlite3_column_type
#define sqlite3_column_value vtksqlite_sqlite3_column_value
#define sqlite3_commit_hook vtksqlite_sqlite3_commit_hook
#define sqlite3_compileoption_get vtksqlite_sqlite3_compileoption_get
#define sqlite3_compileoption_used vtksqlite_sqlite3_compileoption_used
#define sqlite3_complete vtksqlite_sqlite3_complete
#define sqlite3_complete16 vtksqlite_sqlite3_complete16
#define sqlite3_config vtksqlite_sqlite3_config
#define sqlite3_context_db_handle vtksqlite_sqlite3_context_db_handle
#define sqlite3_create_collation vtksqlite_sqlite3_create_collation
#define sqlite3_create_collation16 vtksqlite_sqlite3_create_collation16
#define sqlite3_create_collation_v2 vtksqlite_sqlite3_create_collation_v2
#define sqlite3_create_function vtksqlite_sqlite3_create_function
#define sqlite3_create_function16 vtksqlite_sqlite3_create_function16
#define sqlite3_create_function_v2 vtksqlite_sqlite3_create_function_v2
#define sqlite3_create_module vtksqlite_sqlite3_create_module
#define sqlite3_create_module_v2 vtksqlite_sqlite3_create_module_v2
#define sqlite3_data_count vtksqlite_sqlite3_data_count
#define sqlite3_db_cacheflush vtksqlite_sqlite3_db_cacheflush
#define sqlite3_db_config vtksqlite_sqlite3_db_config
#define sqlite3_db_filename vtksqlite_sqlite3_db_filename
#define sqlite3_db_handle vtksqlite_sqlite3_db_handle
#define sqlite3_db_mutex vtksqlite_sqlite3_db_mutex
#define sqlite3_db_readonly vtksqlite_sqlite3_db_readonly
#define sqlite3_db_release_memory vtksqlite_sqlite3_db_release_memory
#define sqlite3_db_status vtksqlite_sqlite3_db_status
#define sqlite3_declare_vtab vtksqlite_sqlite3_declare_vtab
#define sqlite3_enable_load_extension vtksqlite_sqlite3_enable_load_extension
#define sqlite3_enable_shared_cache vtksqlite_sqlite3_enable_shared_cache
#define sqlite3_errcode vtksqlite_sqlite3_errcode
#define sqlite3_errmsg vtksqlite_sqlite3_errmsg
#define sqlite3_errmsg16 vtksqlite_sqlite3_errmsg16
#define sqlite3_errstr vtksqlite_sqlite3_errstr
#define sqlite3_exec vtksqlite_sqlite3_exec
#define sqlite3_expanded_sql vtksqlite_sqlite3_expanded_sql
#define sqlite3_expired vtksqlite_sqlite3_expired
#define sqlite3_extended_errcode vtksqlite_sqlite3_extended_errcode
#define sqlite3_extended_result_codes vtksqlite_sqlite3_extended_result_codes
#define sqlite3_file_control vtksqlite_sqlite3_file_control
#define sqlite3_finalize vtksqlite_sqlite3_finalize
#define sqlite3_free vtksqlite_sqlite3_free
#define sqlite3_free_table vtksqlite_sqlite3_free_table
#define sqlite3_get_autocommit vtksqlite_sqlite3_get_autocommit
#define sqlite3_get_auxdata vtksqlite_sqlite3_get_auxdata
#define sqlite3_get_table vtksqlite_sqlite3_get_table
#define sqlite3_global_recover vtksqlite_sqlite3_global_recover
#define sqlite3_initialize vtksqlite_sqlite3_initialize
#define sqlite3_interrupt vtksqlite_sqlite3_interrupt
#define sqlite3_last_insert_rowid vtksqlite_sqlite3_last_insert_rowid
#define sqlite3_libversion vtksqlite_sqlite3_libversion
#define sqlite3_libversion_number vtksqlite_sqlite3_libversion_number
#define sqlite3_limit vtksqlite_sqlite3_limit
#define sqlite3_load_extension vtksqlite_sqlite3_load_extension
#define sqlite3_log vtksqlite_sqlite3_log
#define sqlite3_malloc vtksqlite_sqlite3_malloc
#define sqlite3_malloc64 vtksqlite_sqlite3_malloc64
#define sqlite3_memory_alarm vtksqlite_sqlite3_memory_alarm
#define sqlite3_memory_highwater vtksqlite_sqlite3_memory_highwater
#define sqlite3_memory_used vtksqlite_sqlite3_memory_used
#define sqlite3_mprintf vtksqlite_sqlite3_mprintf
#define sqlite3_msize vtksqlite_sqlite3_msize
#define sqlite3_mutex_alloc vtksqlite_sqlite3_mutex_alloc
#define sqlite3_mutex_enter vtksqlite_sqlite3_mutex_enter
#define sqlite3_mutex_free vtksqlite_sqlite3_mutex_free
#define sqlite3_mutex_leave vtksqlite_sqlite3_mutex_leave
#define sqlite3_mutex_try vtksqlite_sqlite3_mutex_try
#define sqlite3_next_stmt vtksqlite_sqlite3_next_stmt
#define sqlite3_open vtksqlite_sqlite3_open
#define sqlite3_open16 vtksqlite_sqlite3_open16
#define sqlite3_open_v2 vtksqlite_sqlite3_open_v2
#define sqlite3_os_end vtksqlite_sqlite3_os_end
#define sqlite3_os_init vtksqlite_sqlite3_os_init
#define sqlite3_overload_function vtksqlite_sqlite3_overload_function
#define sqlite3_prepare vtksqlite_sqlite3_prepare
#define sqlite3_prepare16 vtksqlite_sqlite3_prepare16
#define sqlite3_prepare16_v2 vtksqlite_sqlite3_prepare16_v2
#define sqlite3_prepare16_v3 vtksqlite_sqlite3_prepare16_v3
#define sqlite3_prepare_v2 vtksqlite_sqlite3_prepare_v2
#define sqlite3_prepare_v3 vtksqlite_sqlite3_prepare_v3
#define sqlite3_profile vtksqlite_sqlite3_profile
#define sqlite3_progress_handler vtksqlite_sqlite3_progress_handler
#define sqlite3_randomness vtksqlite_sqlite3_randomness
#define sqlite3_realloc vtksqlite_sqlite3_realloc
#define sqlite3_realloc64 vtksqlite_sqlite3_realloc64
#define sqlite3_release_memory vtksqlite_sqlite3_release_memory
#define sqlite3_reset vtksqlite_sqlite3_reset
#define sqlite3_reset_auto_extension vtksqlite_sqlite3_reset_auto_extension
#define sqlite3_result_blob vtksqlite_sqlite3_result_blob
#define sqlite3_result_blob64 vtksqlite_sqlite3_result_blob64
#define sqlite3_result_double vtksqlite_sqlite3_result_double
#define sqlite3_result_error vtksqlite_sqlite3_result_error
#define sqlite3_result_error16 vtksqlite_sqlite3_result_error16
#define sqlite3_result_error_code vtksqlite_sqlite3_result_error_code
#define sqlite3_result_error_nomem vtksqlite_sqlite3_result_error_nomem
#define sqlite3_result_error_toobig vtksqlite_sqlite3_result_error_toobig
#define sqlite3_result_int vtksqlite_sqlite3_result_int
#define sqlite3_result_int64 vtksqlite_sqlite3_result_int64
#define sqlite3_result_null vtksqlite_sqlite3_result_null
#define sqlite3_result_pointer vtksqlite_sqlite3_result_pointer
#define sqlite3_result_subtype vtksqlite_sqlite3_result_subtype
#define sqlite3_result_text vtksqlite_sqlite3_result_text
#define sqlite3_result_text16 vtksqlite_sqlite3_result_text16
#define sqlite3_result_text16be vtksqlite_sqlite3_result_text16be
#define sqlite3_result_text16le vtksqlite_sqlite3_result_text16le
#define sqlite3_result_text64 vtksqlite_sqlite3_result_text64
#define sqlite3_result_value vtksqlite_sqlite3_result_value
#define sqlite3_result_zeroblob vtksqlite_sqlite3_result_zeroblob
#define sqlite3_result_zeroblob64 vtksqlite_sqlite3_result_zeroblob64
#define sqlite3_rollback_hook vtksqlite_sqlite3_rollback_hook
#define sqlite3_set_authorizer vtksqlite_sqlite3_set_authorizer
#define sqlite3_set_auxdata vtksqlite_sqlite3_set_auxdata
#define sqlite3_set_last_insert_rowid vtksqlite_sqlite3_set_last_insert_rowid
#define sqlite3_shutdown vtksqlite_sqlite3_shutdown
#define sqlite3_sleep vtksqlite_sqlite3_sleep
#define sqlite3_snprintf vtksqlite_sqlite3_snprintf
#define sqlite3_soft_heap_limit vtksqlite_sqlite3_soft_heap_limit
#define sqlite3_soft_heap_limit64 vtksqlite_sqlite3_soft_heap_limit64
#define sqlite3_sourceid vtksqlite_sqlite3_sourceid
#define sqlite3_sql vtksqlite_sqlite3_sql
#define sqlite3_status vtksqlite_sqlite3_status
#define sqlite3_status64 vtksqlite_sqlite3_status64
#define sqlite3_step vtksqlite_sqlite3_step
#define sqlite3_stmt_busy vtksqlite_sqlite3_stmt_busy
#define sqlite3_stmt_readonly vtksqlite_sqlite3_stmt_readonly
#define sqlite3_stmt_status vtksqlite_sqlite3_stmt_status
#define sqlite3_strglob vtksqlite_sqlite3_strglob
#define sqlite3_stricmp vtksqlite_sqlite3_stricmp
#define sqlite3_strlike vtksqlite_sqlite3_strlike
#define sqlite3_strnicmp vtksqlite_sqlite3_strnicmp
#define sqlite3_system_errno vtksqlite_sqlite3_system_errno
#define sqlite3_table_column_metadata vtksqlite_sqlite3_table_column_metadata
#define sqlite3_test_control vtksqlite_sqlite3_test_control
#define sqlite3_thread_cleanup vtksqlite_sqlite3_thread_cleanup
#define sqlite3_threadsafe vtksqlite_sqlite3_threadsafe
#define sqlite3_total_changes vtksqlite_sqlite3_total_changes
#define sqlite3_trace vtksqlite_sqlite3_trace
#define sqlite3_trace_v2 vtksqlite_sqlite3_trace_v2
#define sqlite3_transfer_bindings vtksqlite_sqlite3_transfer_bindings
#define sqlite3_update_hook vtksqlite_sqlite3_update_hook
#define sqlite3_uri_boolean vtksqlite_sqlite3_uri_boolean
#define sqlite3_uri_int64 vtksqlite_sqlite3_uri_int64
#define sqlite3_uri_parameter vtksqlite_sqlite3_uri_parameter
#define sqlite3_user_data vtksqlite_sqlite3_user_data
#define sqlite3_value_blob vtksqlite_sqlite3_value_blob
#define sqlite3_value_bytes vtksqlite_sqlite3_value_bytes
#define sqlite3_value_bytes16 vtksqlite_sqlite3_value_bytes16
#define sqlite3_value_double vtksqlite_sqlite3_value_double
#define sqlite3_value_dup vtksqlite_sqlite3_value_dup
#define sqlite3_value_free vtksqlite_sqlite3_value_free
#define sqlite3_value_int vtksqlite_sqlite3_value_int
#define sqlite3_value_int64 vtksqlite_sqlite3_value_int64
#define sqlite3_value_numeric_type vtksqlite_sqlite3_value_numeric_type
#define sqlite3_value_pointer vtksqlite_sqlite3_value_pointer
#define sqlite3_value_subtype vtksqlite_sqlite3_value_subtype
#define sqlite3_value_text vtksqlite_sqlite3_value_text
#define sqlite3_value_text16 vtksqlite_sqlite3_value_text16
#define sqlite3_value_text16be vtksqlite_sqlite3_value_text16be
#define sqlite3_value_text16le vtksqlite_sqlite3_value_text16le
#define sqlite3_value_type vtksqlite_sqlite3_value_type
#define sqlite3_version vtksqlite_sqlite3_version
#define sqlite3_vfs_find vtksqlite_sqlite3_vfs_find
#define sqlite3_vfs_register vtksqlite_sqlite3_vfs_register
#define sqlite3_vfs_unregister vtksqlite_sqlite3_vfs_unregister
#define sqlite3_vmprintf vtksqlite_sqlite3_vmprintf
#define sqlite3_vsnprintf vtksqlite_sqlite3_vsnprintf
#define sqlite3_vtab_config vtksqlite_sqlite3_vtab_config
#define sqlite3_vtab_on_conflict vtksqlite_sqlite3_vtab_on_conflict
#define sqlite3_wal_autocheckpoint vtksqlite_sqlite3_wal_autocheckpoint
#define sqlite3_wal_checkpoint vtksqlite_sqlite3_wal_checkpoint
#define sqlite3_wal_checkpoint_v2 vtksqlite_sqlite3_wal_checkpoint_v2
#define sqlite3_wal_hook vtksqlite_sqlite3_wal_hook

#endif
