#ifndef vtk_theora_mangle_h
#define vtk_theora_mangle_h

#define th_comment_add vtktheora_th_comment_add
#define th_comment_add_tag vtktheora_th_comment_add_tag
#define th_comment_clear vtktheora_th_comment_clear
#define th_comment_init vtktheora_th_comment_init
#define th_comment_query vtktheora_th_comment_query
#define th_comment_query_count vtktheora_th_comment_query_count
#define th_decode_alloc vtktheora_th_decode_alloc
#define th_decode_ctl vtktheora_th_decode_ctl
#define th_decode_free vtktheora_th_decode_free
#define th_decode_headerin vtktheora_th_decode_headerin
#define th_decode_packetin vtktheora_th_decode_packetin
#define th_decode_ycbcr_out vtktheora_th_decode_ycbcr_out
#define th_encode_alloc vtktheora_th_encode_alloc
#define th_encode_ctl vtktheora_th_encode_ctl
#define th_encode_flushheader vtktheora_th_encode_flushheader
#define th_encode_free vtktheora_th_encode_free
#define th_encode_packetout vtktheora_th_encode_packetout
#define th_encode_ycbcr_in vtktheora_th_encode_ycbcr_in
#define theora_clear vtktheora_theora_clear
#define theora_comment_add vtktheora_theora_comment_add
#define theora_comment_add_tag vtktheora_theora_comment_add_tag
#define theora_comment_clear vtktheora_theora_comment_clear
#define theora_comment_init vtktheora_theora_comment_init
#define theora_comment_query vtktheora_theora_comment_query
#define theora_comment_query_count vtktheora_theora_comment_query_count
#define theora_control vtktheora_theora_control
#define theora_decode_header vtktheora_theora_decode_header
#define theora_decode_init vtktheora_theora_decode_init
#define theora_decode_packetin vtktheora_theora_decode_packetin
#define theora_decode_YUVout vtktheora_theora_decode_YUVout
#define theora_encode_comment vtktheora_theora_encode_comment
#define theora_encode_header vtktheora_theora_encode_header
#define theora_encode_init vtktheora_theora_encode_init
#define theora_encode_packetout vtktheora_theora_encode_packetout
#define theora_encode_tables vtktheora_theora_encode_tables
#define theora_encode_YUVin vtktheora_theora_encode_YUVin
#define theora_granule_frame vtktheora_theora_granule_frame
#define theora_granule_time vtktheora_theora_granule_time
#define theora_info_clear vtktheora_theora_info_clear
#define theora_info_init vtktheora_theora_info_init
#define theora_packet_isheader vtktheora_theora_packet_isheader
#define theora_packet_iskeyframe vtktheora_theora_packet_iskeyframe
#define theora_version_number vtktheora_theora_version_number
#define theora_version_string vtktheora_theora_version_string
#define th_granule_frame vtktheora_th_granule_frame
#define th_granule_time vtktheora_th_granule_time
#define th_info_clear vtktheora_th_info_clear
#define th_info_init vtktheora_th_info_init
#define th_packet_isheader vtktheora_th_packet_isheader
#define th_packet_iskeyframe vtktheora_th_packet_iskeyframe
#define th_setup_free vtktheora_th_setup_free
#define th_version_number vtktheora_th_version_number
#define th_version_string vtktheora_th_version_string

#endif
