"""PyQt v4 widget for VTK.

This module is a version of the vtkmodules.qt module that requires Qt4.
If you do not specifically require Qt4, then use vtkmodules.qt instead.
"""

__all__ = ['QVTKRenderWindowInteractor']
